@@ -1,3 +1,4 @@
+import logging
 import re
 from dataclasses import dataclass, field
 from operator import itemgetter
@@ -8,6 +9,8 @@ from dulwich.objects import Commit as RawCommit
 from dulwich.repo import Repo
 from semver import VersionInfo as Version
 
+logger = logging.getLogger(__name__)
+
 # Default version tag regex. It is used to find the last valid git tag.
 # This regex will match the following tags: "v1", "v1.0", "v1.0.0"
 TAG_REGEX = re.compile(
@@ -99,42 +102,46 @@ class Git:
         """
 
         try:
-            walker = self.repo.get_walker(reverse=True)  # type: ignore
+            # Determine include and exclude parameters for proper revision range
+            include = None
+            exclude = None
+
+            if to_tag is not None:
+                # Include commits reachable from to_tag
+                to_sha = self.repo.get_peeled(to_tag)
+                include = [to_sha]
+            else:
+                # Default to HEAD if no to_tag specified
+                include = [self.repo.head()]
+
+            if from_tag is not None:
+                # Exclude commits reachable from from_tag
+                from_sha = self.repo.get_peeled(from_tag)
+                exclude = [from_sha]
+
+            logger.debug("Going to search commits include=%s, exclude=%s", include, exclude)
+            # Get walker with proper include/exclude to implement revision range
+            walker = self.repo.get_walker(include=include, exclude=exclude, reverse=True)
         except KeyError:  # Repo is empty
+            logger.debug("Got error, returning empty list.")
             return []
 
-        # Convert from_ref to SHA if it is a tag name
-        from_sha = self.repo.get_peeled(from_tag) if from_tag else None  # type: ignore
-
-        # Convert to_ref to SHA if it is a tag name
-        to_sha = self.repo.get_peeled(to_tag) if to_tag else None  # type: ignore
-
-        if from_sha is None:
-            add = True
-        else:
-            add = False
         commits: List[Commit] = []
         for entry in walker:
             commit: RawCommit = entry.commit
-            hash = commit.id
-
-            if add:
-                message = commit.message.decode()
-                subject, body = parse_message(message)
-                paths = self.get_commit_paths(commit)
-                commits.append(Commit(hash, subject, body or None, paths))
-
-            if hash == from_sha:
-                add = True
-            if to_tag and hash == to_sha:
-                break
+            message = commit.message.decode()
+            subject, body = parse_message(message)
+            paths = self.get_commit_paths(commit)
+            c = Commit(commit.id, subject, body or None, paths)
+            logger.debug("Found commit=%s", c)
+            commits.append(c)
 
         return commits
 
     def retrieve_last_version(
         self, scope: Optional[str] = None
     ) -> Tuple[Optional[bytes], Optional[Version]]:
-        """Retrieve last valid version from a tag. Any non-valid version tags are skipped.
+        """Retrieve the last valid version from a tag. Any non-valid version tags are skipped.
         Return a tuple with tag name and version or None.
 
         If `scope` is not None, only consider tags with this scope."""
@@ -154,9 +161,9 @@ class Git:
                         (
                             tag,
                             Version(
-                                match_dict["major"],
-                                match_dict["minor"] or 0,
-                                match_dict["patch"] or 0,
+                                match_dict["major"],  # type: ignore
+                                match_dict["minor"] or 0,  # type: ignore
+                                match_dict["patch"] or 0,  # type: ignore
                             ),
                         )
                     )
@@ -166,4 +173,4 @@ class Git:
         if sorted_tags:
             return sorted_tags[-1]
         else:
-            return None, None
\ No newline at end of file
+            return None, None
